@@ -20,6 +20,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+#ident "$Id: insmod.c,v 1.1 1997/09/10 22:14:47 rth Exp $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -40,7 +41,7 @@
 
 #include "logger.h"
 
-#define version_parts(x)  ((x)>>16) & 0xff, ((x)>>8) & 0xff, (x) & 0xff
+#define STRVERSIONLEN	32
 
 
 /*======================================================================*/
@@ -85,6 +86,7 @@ search_module_path(char *base)
     "/lib/modules/%s/block:"
     "/lib/modules/%s/cdrom:"
     "/lib/modules/%s/ipv4:"
+    "/lib/modules/%s/ipv6:"
     "/lib/modules/%s/misc:"
     "/lib/modules/default/fs:"
     "/lib/modules/default/net:"
@@ -92,6 +94,7 @@ search_module_path(char *base)
     "/lib/modules/default/block:"
     "/lib/modules/default/cdrom:"
     "/lib/modules/default/ipv4:"
+    "/lib/modules/default/ipv6:"
     "/lib/modules/default/misc:"
     "/lib/modules/fs:"
     "/lib/modules/net:"
@@ -99,6 +102,7 @@ search_module_path(char *base)
     "/lib/modules/block:"
     "/lib/modules/cdrom:"
     "/lib/modules/ipv4:"
+    "/lib/modules/ipv6:"
     "/lib/modules/misc";
 
   char *path, *p, *filename;
@@ -139,14 +143,15 @@ search_module_path(char *base)
 /* Get the kernel version in the canonical integer form.  */
 
 static int
-get_kernel_version(void)
+get_kernel_version(char str[STRVERSIONLEN])
 {
   struct utsname uts_info;
-  char *p;
+  char *p, *q;
   int a, b, c;
 
   if (uname(&uts_info) < 0)
     return -1;
+  strncpy(str, uts_info.release, STRVERSIONLEN);
   p = uts_info.release;
 
   a = strtoul(p, &p, 10);
@@ -155,35 +160,24 @@ get_kernel_version(void)
   b = strtoul(p+1, &p, 10);
   if (*p != '.')
     return -1;
-  c = strtoul(p+1, &p, 10);
-  if (*p != '\0')
+  c = strtoul(p+1, &q, 10);
+  if (p+1 == q)
     return -1;
 
   return a << 16 | b << 8 | c;
 }
 
-/* String comparison for unversioned kernel, versioned module.  */
+/* String comparison for non-co-versioned kernel and module.  */
 
 static int
-m_strcmp(const char *a, const char *b)
-{
-  size_t alen = strlen(a), blen = strlen(b);
-
-  if (alen == blen + 10 && a[blen] == '_' && a[blen+1] == 'R')
-    return strncmp(a, b, blen);
-  else
-    return strcmp(a, b);
-}
-
-/* String comparison for versioned kernel, unversioned module.  */
-
-static int
-k_strcmp(const char *a, const char *b)
+ncv_strcmp(const char *a, const char *b)
 {
   size_t alen = strlen(a), blen = strlen(b);
 
   if (blen == alen + 10 && b[alen] == '_' && b[alen+1] == 'R')
     return strncmp(a, b, alen);
+  else if (alen == blen + 10 && a[blen] == '_' && a[blen+1] == 'R')
+    return strncmp(a, b, blen);
   else
     return strcmp(a, b);
 }
@@ -478,16 +472,17 @@ old_is_kernel_checksummed(void)
 /* Get the module's kernel version in the canonical integer form.  */
 
 static int
-old_get_module_version(struct obj_file *f)
+old_get_module_version(struct obj_file *f, char str[STRVERSIONLEN])
 {
   struct obj_symbol *sym;
-  char *p;
+  char *p, *q;
   int a, b, c;
 
   sym = obj_find_symbol(f, "kernel_version");
   if (sym == NULL)
     return -1;
   p = f->sections[sym->secidx]->contents + sym->value;
+  strncpy(str, p, STRVERSIONLEN);
 
   a = strtoul(p, &p, 10);
   if (*p != '.')
@@ -495,8 +490,8 @@ old_get_module_version(struct obj_file *f)
   b = strtoul(p+1, &p, 10);
   if (*p != '.')
     return -1;
-  c = strtoul(p+1, &p, 10);
-  if (*p != '\0')
+  c = strtoul(p+1, &q, 10);
+  if (p+1 == q)
     return -1;
 
   return a << 16 | b << 8 | c;
@@ -552,7 +547,23 @@ old_process_module_arguments(struct obj_file *f, int argc, char **argv)
 	  while (*q++ == ',');
 	}
       else
-	obj_string_patch(f, sym->secidx, sym->value, q);
+	{
+	  char *contents = f->sections[sym->secidx]->contents;
+	  char *loc = contents + sym->value;
+	  char *r;	/* To search for commas */
+
+	  /* Break the string with comas */
+	  while((r = strchr(q, ',')) != (char *) NULL)
+	    {
+	      *r++ = '\0';
+	      obj_string_patch(f, sym->secidx, loc-contents, q);
+	      loc += sizeof(char*);
+	      q = r;
+	    }
+
+	  /* last part */
+	  obj_string_patch(f, sym->secidx, loc-contents, q);
+	}
 
       argc--, argv++;
     }
@@ -818,14 +829,15 @@ get_modinfo_value(struct obj_file *f, const char *key)
 /* Get the module's kernel version in the canonical integer form.  */
 
 static int
-new_get_module_version(struct obj_file *f)
+new_get_module_version(struct obj_file *f, char str[STRVERSIONLEN])
 {
-  char *p;
+  char *p, *q;
   int a, b, c;
 
   p = get_modinfo_value(f, "kernel_version");
   if (p == NULL)
     return -1;
+  strncpy(str, p, STRVERSIONLEN);
 
   a = strtoul(p, &p, 10);
   if (*p != '.')
@@ -833,8 +845,8 @@ new_get_module_version(struct obj_file *f)
   b = strtoul(p+1, &p, 10);
   if (*p != '.')
     return -1;
-  c = strtoul(p+1, &p, 10);
-  if (*p != '\0')
+  c = strtoul(p+1, &q, 10);
+  if (p+1 == q)
     return -1;
 
   return a << 16 | b << 8 | c;
@@ -901,7 +913,7 @@ new_process_module_arguments(struct obj_file *f, int argc, char **argv)
 
       while (1)
 	{
-	  if (*p == 's')
+	  if((*p == 's') || (*p == 'c'))
 	    {
 	      char *str;
 
@@ -955,12 +967,70 @@ new_process_module_arguments(struct obj_file *f, int argc, char **argv)
 		}
 	      else
 		{
-		  str = q;
-		  q = "";
+		  char *r;
+
+		  /* In this case, the string is not quoted. We will break
+		     it using the coma (like for ints). If the user wants to
+		     include comas in a string, he just has to quote it */
+
+		  /* Search the next coma */
+		  r = strchr(q, ',');
+
+		  /* Found ? */
+		  if(r != (char *) NULL)
+		    {
+		      /* Recopy the current field */
+		      str = alloca(r - q + 1);
+		      memcpy(str, q, r - q);
+
+		      /* I don't know if it is usefull, as the previous case
+		         doesn't null terminate the string ??? */
+		      str[r - q] = '\0';
+
+		      /* Keep next fields */
+		      q = r;
+		    }
+		  else
+		    {
+		      /* last string */
+		      str = q;
+		      q = "";
+		    }
 		}
 
-	      obj_string_patch(f, sym->secidx, loc-contents, str);
-	      loc += sizeof(char*);
+	      if (*p == 's')
+		{
+		  /* Normal string */
+		  obj_string_patch(f, sym->secidx, loc-contents, str);
+		  loc += tgt_sizeof_char_p;
+		}
+	      else
+		{
+		  /* Array of chars (in fact, matrix !) */
+		  long	charssize;	/* size of each member */
+
+		  /* Get the size of each member */
+		  /* Probably we should do that outside the loop ? */
+		  if(!isdigit(*(p + 1)))
+		    {
+		      error("parameter type 'c' for %s must be followed by the maximum size",
+			    key);
+		      return 0;
+		    }
+		  charssize = strtoul(p + 1, (char **) NULL, 10);
+
+		  /* Check length */
+		  if(strlen(str) >= charssize)
+		    {
+		      error("string too long for %s (max %ld)",
+			    key, charssize - 1);
+		      return 0;
+		    }
+
+		  /* Copy to location */
+		  strcpy((char *) loc, str);
+		  loc += charssize;
+		}
 	    }
 	  else
 	    {
@@ -972,15 +1042,15 @@ new_process_module_arguments(struct obj_file *f, int argc, char **argv)
 		  break;
 		case 'h':
 		  *(short *)loc = v;
-		  loc += sizeof(short);
+		  loc += tgt_sizeof_short;
 		  break;
 		case 'i':
 		  *(int *)loc = v;
-		  loc += sizeof(int);
+		  loc += tgt_sizeof_int;
 		  break;
 		case 'l':
 		  *(long *)loc = v;
-		  loc += sizeof(long);
+		  loc += tgt_sizeof_long;
 		  break;
 
 		default:
@@ -1033,7 +1103,7 @@ new_create_this_module(struct obj_file *f, const char *m_name)
 {
   struct obj_section *sec;
 
-  sec = obj_create_alloced_section_first(f, ".this", sizeof(long),
+  sec = obj_create_alloced_section_first(f, ".this", tgt_sizeof_long,
 					 sizeof(struct new_module));
   memset(sec->contents, 0, sizeof(struct new_module));
 
@@ -1058,7 +1128,7 @@ new_create_module_ksymtab(struct obj_file *f)
       struct new_module_ref *dep;
       struct obj_symbol *tm;
 
-      sec = obj_create_alloced_section(f, ".kmodtab", sizeof(void*),
+      sec = obj_create_alloced_section(f, ".kmodtab", tgt_sizeof_void_p,
 				       (sizeof(struct new_module_ref)
 					* n_ext_modules_used));
       if (!sec)
@@ -1081,7 +1151,7 @@ new_create_module_ksymtab(struct obj_file *f)
       size_t nsyms;
       int *loaded;
 
-      sec = obj_create_alloced_section(f, "__ksymtab", sizeof(void*), 0);
+      sec = obj_create_alloced_section(f, "__ksymtab", tgt_sizeof_void_p, 0);
 
       /* We don't want to export symbols residing in sections that
 	 aren't loaded.  There are a number of these created so that
@@ -1100,16 +1170,16 @@ new_create_module_ksymtab(struct obj_file *f)
 		&& (sym->secidx >= SHN_LORESERVE
 		    || loaded[sym->secidx]))
 	      {
-		ElfW(Addr) ofs = nsyms * 2*sizeof(void*);
+		ElfW(Addr) ofs = nsyms * 2*tgt_sizeof_void_p;
 
 		obj_symbol_patch(f, sec->idx, ofs, sym);
-		obj_string_patch(f, sec->idx, ofs+sizeof(void*), sym->name);
+		obj_string_patch(f, sec->idx, ofs+tgt_sizeof_void_p, sym->name);
 
 		nsyms++;
 	      }
 	}
 
-      obj_extend_section(sec, nsyms * 2 * sizeof(char*));
+      obj_extend_section(sec, nsyms * 2 * tgt_sizeof_char_p);
     }
 
   return 1;
@@ -1134,7 +1204,7 @@ new_init_module(const char *m_name, struct obj_file *f, unsigned long m_size)
   if (sec && sec->header.sh_size)
     {
       module->syms = sec->header.sh_addr;
-      module->nsyms = sec->header.sh_size / (2 * sizeof(char*));
+      module->nsyms = sec->header.sh_size / (2 * tgt_sizeof_char_p);
     }
 
   if (n_ext_modules_used)
@@ -1182,6 +1252,7 @@ main(int argc, char **argv)
   int k_version;
   int k_crcs;
   int k_new_syscalls;
+  char k_strversion[STRVERSIONLEN];
 
   char *m_name = NULL;
   int m_version;
@@ -1189,6 +1260,7 @@ main(int argc, char **argv)
   unsigned long m_size;
   int m_crcs;
   int m_has_modinfo;
+  char m_strversion[STRVERSIONLEN];
 
   char *filename;
   FILE *fp;
@@ -1311,14 +1383,14 @@ main(int argc, char **argv)
 
   /* Version correspondence?  */
 
-  k_version = get_kernel_version();
-  m_version = new_get_module_version(f);
+  k_version = get_kernel_version(k_strversion);
+  m_version = new_get_module_version(f, m_strversion);
   if (m_version != -1)
     m_has_modinfo = 1;
   else
     {
       m_has_modinfo = 0;
-      m_version = old_get_module_version(f);
+      m_version = old_get_module_version(f, m_strversion);
       if (m_version == -1)
 	{
 	  error("couldn't find the kernel version the module was compiled for");
@@ -1326,23 +1398,21 @@ main(int argc, char **argv)
 	}
     }
 
-  if (k_version != m_version)
+  if (strncmp(k_strversion, m_strversion, STRVERSIONLEN) != 0)
     {
       if (flag_force_load)
 	{
 	  lprintf("Warning: kernel-module version mismatch\n"
-		  "\t%s was compiled for kernel version %d.%d.%d\n"
-		  "\twhile this kernel is version %d.%d.%d.",
-		  filename, version_parts(m_version),
-		  version_parts(k_version));
+		  "\t%s was compiled for kernel version %s\n"
+		  "\twhile this kernel is version %s\n",
+		  filename, m_strversion, k_strversion);
 	}
       else
 	{
 	  error("kernel-module version mismatch\n"
-		"\t%s was compiled for kernel version %d.%d.%d\n"
-		"\twhile this kernel is version %d.%d.%d.",
-		filename, version_parts(m_version),
-		version_parts(k_version));
+		"\t%s was compiled for kernel version %s\n"
+		"\twhile this kernel is version %s.",
+		filename, m_strversion, k_strversion);
 	  return 1;
 	}
     }
@@ -1367,10 +1437,8 @@ main(int argc, char **argv)
   else
     m_crcs = old_is_module_checksummed(f);
 
-  if (m_crcs && !k_crcs)
-    obj_set_symbol_compare(f, m_strcmp, ncv_symbol_hash);
-  else if (!m_crcs && k_crcs)
-    obj_set_symbol_compare(f, k_strcmp, ncv_symbol_hash);
+  if (m_crcs != k_crcs)
+    obj_set_symbol_compare(f, ncv_strcmp, ncv_symbol_hash);
 
   /* Let the module know about the kernel symbols.  */
 
