@@ -1,29 +1,3 @@
-#define CONFIG_KERNELD
-#include <linux/kerneld.h>
-#include <stdio.h>
-#include <stdarg.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/time.h>
-#include <sys/ipc.h>
-#include <sys/msg.h>
-#include <sys/wait.h>
-#include <sys/ioctl.h>
-#include <signal.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <syslog.h>
-#include <linux/unistd.h>
-#include <asm/param.h>
-#include <signal.h>
-#include <errno.h>
-#include <sys/utsname.h>
-
-#include "version.h"
-#include "module.h"
-
 /*
  * This is kerneld, the user level handler of kernel requests.
  * Copyright (C) 1995, 1996 Bjorn Ekwall <bj0rn@blox.se>
@@ -52,6 +26,34 @@
  * Better error reports by: Lars Fenneberg <in5y050@public.uni-hamburg.de>
  */
 
+#ident "$Id: kerneld.c,v 1.1 1997/09/10 22:16:32 rth Exp $"
+
+#define CONFIG_KERNELD
+#include <linux/kerneld.h>
+#include <stdio.h>
+#include <stdarg.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/ipc.h>
+#include <sys/msg.h>
+#include <sys/wait.h>
+#include <sys/ioctl.h>
+#include <signal.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <syslog.h>
+#include <linux/unistd.h>
+#include <asm/param.h>
+#include <signal.h>
+#include <errno.h>
+#include <sys/utsname.h>
+
+#include "version.h"
+#include "module.h"
+
 #define MAXCMD 255
 /*
  * The _real_ maximum message type is KERNELD_MAXCMD,
@@ -278,26 +280,6 @@ void kerneld_perror (char *name)
 	exit (1);
 }
 
-void handle_child(int sig);
-struct sigaction child_action = {
-	handle_child,
-	0,
-	SA_RESTART | SA_NOMASK,
-};
-
-void handle_timer(int sig);
-struct sigaction timer_action = {
-	handle_timer,
-	0,
-	SA_RESTART | SA_NOMASK,
-};
-
-struct sigaction hup_action = {
-	gdbm_reload,
-	0,
-	SA_RESTART | SA_NOMASK,
-};
-
 volatile int timer = 0;
 extern int errno;
 
@@ -793,8 +775,9 @@ main(int argc, char *argv[])
 	struct job *job;
 	int pid;
 	int status;
-	sigset_t signalset;
 	struct utsname uts_info;
+	struct sigaction sa;
+	sigset_t hold_signals;
 
 	/* make me a daemon */
 	if ((pid = fork()) != 0) {
@@ -821,13 +804,25 @@ main(int argc, char *argv[])
 
 	setsid();
 
-	if (sigaction(SIGCHLD, &child_action, NULL) < 0) {
+	sigemptyset(&hold_signals);
+	sigaddset(&hold_signals, SIGCHLD);
+	sigaddset(&hold_signals, SIGHUP);
+
+	sa.sa_mask = hold_signals;
+	sa.sa_flags = SA_RESTART;
+
+	sa.sa_handler = handle_child;
+	if (sigaction(SIGCHLD, &sa, NULL) < 0) {
 		kerneld_perror("sigaction(SIGCHLD)");
 	}
-	if (sigaction(SIGALRM, &timer_action, NULL) < 0) {
+
+	sa.sa_handler = handle_timer;
+	if (sigaction(SIGALRM, &sa, NULL) < 0) {
 		kerneld_perror("sigaction(SIGARLM)");
 	}
-	if (sigaction(SIGHUP, &hup_action, NULL) < 0) {
+
+	sa.sa_handler = gdbm_reload;
+	if (sigaction(SIGHUP, &sa, NULL) < 0) {
 		kerneld_perror("sigaction(SIGHUP)");
 	}
 
@@ -884,10 +879,7 @@ main(int argc, char *argv[])
 			type2text((int)job->msg.mtype), job->msg.text));
 
 		/* Don't trap SIGCHLD til we add the job to the queue */
-		sigemptyset(&signalset);
-		sigaddset(&signalset, SIGCHLD);
-		sigaddset(&signalset, SIGHUP);
-		sigprocmask(SIG_BLOCK, &signalset, NULL);
+		sigprocmask(SIG_BLOCK, &hold_signals, NULL);
 		if ((pid = spawn_it(job)) < 0)
 			break; /* fatal error */
 
@@ -908,7 +900,7 @@ main(int argc, char *argv[])
 			DPRINT(("job (%08lx) released\n", (long)job));
 			free(job);
 		}
-		sigprocmask(SIG_UNBLOCK, &signalset, NULL);
+		sigprocmask(SIG_UNBLOCK, &hold_signals, NULL);
 	}
 
 	kerneld_perror("exit");
